@@ -12,10 +12,12 @@
 tglkmeans.set_parallel <- function(thread_num) {
     if (thread_num <= 1) {
         options(tglkmeans.parallel = FALSE)
+        RcppParallel::setThreadOptions(numThreads = 1)
     } else {
         doFuture::registerDoFuture()
         future::plan(future::multicore, workers = thread_num)
         options(tglkmeans.parallel = TRUE)
         options(tglkmeans.parallel.thread_num = thread_num)
+        RcppParallel::setThreadOptions(numThreads = thread_num)
     }
 }
